@@ -45,6 +45,18 @@ class PandaHubClient:
     def get_projects(self):
         return self._post("/projects/get_projects").json()
 
+    def get_project_settings(self):
+        return self._post("/projects/get_project_settings").json()
+
+    def set_project_settings(self, settings):
+        return self._post("/projects/set_project_settings", json=locals())
+
+    def get_project_metadata(self):
+        return self._post("/projects/get_project_metadata").json()
+
+    def set_project_metadata(self, metadata):
+        return self._post("/projects/set_project_metadata", json=locals())
+
     ### NET HANDLING
 
     def write_network_to_db(self, net, name, overwrite=True):
@@ -57,6 +69,24 @@ class PandaHubClient:
         return pp.from_json_string(r.json())
 
 
+    ### ELEMENT HANDLING
+
+    def get_net_value_from_db(self, net_name, element, element_index, parameter):
+        return self._post("/net/get_net_value_from_db", json=locals()).json()
+
+    def set_net_value_in_db(self, net_name, element, element_index, parameter, value):
+        return self._post("/net/set_net_value_in_db", json=locals())
+
+    def create_element_in_db(self, net_name, element, element_index, data):
+        return self._post("/net/create_element_in_db", json=locals())
+
+    def create_elements_in_db(self, net_name, element_type, elements_data):
+        return self._post("/net/create_elements_in_db", json=locals())
+
+    def delete_net_element(self, net_name, element, element_index):
+        return self._post("/net/delete_net_element", json=locals())
+
+
     ### TIMESERIES
 
     def multi_get_timeseries_from_db(self, filter_document={}, timestamp_range=None,
@@ -80,6 +110,8 @@ class PandaHubClient:
         json["timeseries"] = json["timeseries"].to_json(date_format="iso")
         return self._post("/timeseries/write_timeseries_to_db", json=json)
 
+
+
     ### INTERNAL
 
     def _post(self, path, json=None, authorize=True):
